@@ -27,12 +27,26 @@ static inline int furwock_read_down(struct furwock *rwock)
 
 	ret = futex_down(&rwock->gate);
 	if (ret == 0) {
-		__furwock_inc(&rwock->count);
+		__atomic_inc(&rwock->count);
 		futex_up(&rwock->gate);
 	}
 	return ret;
 }
 
+static inline int furwock_read_trydown(struct furwock *rwock)
+{
+	int ret;
+
+	ret = futex_trydown(&rwock->gate);
+	if (ret == 0) {
+		if(rwock->count >= 0)
+			__atomic_inc(&rwock->count);
+		futex_up(&rwock->gate);
+	}
+	return ret;
+}
+
+
 static inline void furwock_read_up(struct furwock *rwock)
 {
 	/* Last one out wakes and writer waiting. */
@@ -53,6 +67,19 @@ static inline int furwock_write_down(struct furwock *rwock)
 	return ret;
 }
 
+static inline int furwock_write_trydown(struct furwock *rwock)
+{
+	int ret;
+
+	ret = futex_trydown(&rwock->gate);
+	if (ret == 0) {
+		if (__furwock_dec_negative(&rwock->count))
+			return ret;
+		futex_down(&rwock->wait);
+	}
+	return ret;
+}
+
 static inline void furwock_write_up(struct furwock *rwock)
 {
 	rwock->count = 0;
