@@ -1,12 +1,15 @@
 /* (C) Rusty Russell <rusty@rustcorp.com.au> */
 /* Stolen from Linux kernel (Paul Mackerras) */
 
-/* Atomic dec: return 1 unless counter decremented to zero.
- */
-static __inline__ int __futex_down(int *c)
+/* Atomic dec: return new value. */
+static __inline__ int __futex_down(int *counter)
 {
 	int val;
 
+	/* Don't decrement if already negative. */
+	if (*counter < 0)
+		return *counter;
+
 	/* The isync here stops the chip from pulling loads inside the
            lock above this code */
 	__asm__ __volatile__(
@@ -16,47 +19,51 @@ static __inline__ int __futex_down(int *c)
 	bne-	1b\n\
 	isync"
 	: "=&r" (val)
-	: "r" (c)
+	: "r" (counter)
 	: "cc", "memory");
 
-	return (val != 0);
+	return val;
 }
 
-/* Atomic inc: return 1 unless counter incremented from 0 to 1. */
-static __inline__ int __futex_up(int *c)
+/* Atomic inc: return 1 if counter incremented from 0 to 1. */
+static __inline__ int __futex_up(int *counter)
 {
 	int val;
 
+	/* sync makes sure reads AND writes are committed before we
+           unlock. */
 	__asm__ __volatile__(
-"1:	lwarx	%0,0,%1		# __up\n\
+"	sync\n\
+1:	lwarx	%0,0,%1		# __up\n\
 	addic	%0,%0,1\n\
 	stwcx.	%0,0,%1\n\
-	bne-	1b\n\
-	isync"
+	bne-	1b"
 	: "=&r" (val)
-	: "r" (c)
+	: "r" (counter)
 	: "cc", "memory");
 
-	return (val != 1);
+	return (val == 1);
 }
 
-static __inline__ void __furwock_inc(int *c)
+/* Simple atomic inc. */
+static __inline__ void __atomic_inc(int *counter)
 {
-	int t;
+	int val;
 
 	__asm__ __volatile__(
 "1:	lwarx	%0,0,%2		# __furwock_inc\n\
 	addic	%0,%0,1\n"
 "	stwcx.	%0,0,%2 \n\
 	bne-	1b"
-	: "=&r" (t), "=m" (*c)
-	: "r" (c), "m" (*c)
+	: "=&r" (val), "=m" (*counter)
+	: "r" (counter), "m" (*counter)
 	: "cc");
 }
 
-static __inline__ int __furwock_dec_negative(int *c)
+/* Atomic dec: return 1 if result is negative */
+static __inline__ int __furwock_dec_negative(int *counter)
 {
-	int t;
+	int val;
 
 	__asm__ __volatile__(
 "1:	lwarx	%0,0,%1		# __furwock_dec_negative\n\
@@ -64,11 +71,11 @@ static __inline__ int __furwock_dec_negative(int *c)
 "	stwcx.	%0,0,%1\n\
 	bne-	1b"
 	"\n\tisync"
-	: "=&r" (t)
-	: "r" (c)
+	: "=&r" (val)
+	: "r" (counter)
 	: "cc", "memory");
 
-	return t < 0;
+	return val < 0;
 }
 
 /* Commit the write, so it happens before we send the semaphore to
