@@ -36,18 +36,28 @@ int main(int argc, char *argv[])
 	sem = map + atoi(argv[2]);
 
 	for (i = 0; i < atoi(argv[3]); i++) {
+		int j;
+
+		for (j = random() % 1024; j > 0; j--);
+
 		fprintf(stderr, "%i", getpid());
 		futex_down(sem);
 		fprintf(stderr, "v\n");
 		/* Write pid here */
 		*(volatile int *)(sem + 1) = getpid();
-		sleep(0);
-		if (*(volatile int *)(sem + 1) != getpid())
-			printf("ARGH!\n");
+		for (j = random() % 4096; j > 0; j--) {
+			if (*(volatile int *)(sem + 1) != getpid()) {
+				printf("%u: ARGH!\n", getpid());
+				exit(1);
+			}
+		}
 		fprintf(stderr, "%i", getpid());
+#if 1
 		futex_up(sem);
+#else
+		futex_up_fair(sem);
+#endif
 		fprintf(stderr, "^\n");
-		sleep(0);
 	}
 	exit(0);
 }
