@@ -14,7 +14,7 @@ int main(int argc, char *argv[])
 	void *map;
 
 	if (argc != 4) {
-		fprintf(stderr, "Usage: test <file> <offset> show|init|up|down|mem\n");
+		fprintf(stderr, "Usage: test <file> <offset> show|init|up|upfair|down|down1sec|mem\n");
 		exit(1);
 	}
 	fd = open(argv[1], O_RDWR);
@@ -36,15 +36,34 @@ int main(int argc, char *argv[])
 	if (strcmp(argv[3], "init") == 0)
 		futex_init(futex);
 	else if (strcmp(argv[3], "up") == 0) {
-		if (futex_up(futex) != 0) {
+		int woken;
+		woken = futex_up(futex);
+		if (woken < 0) {
 			perror("futex_up");
 			exit(1);
 		}
+		printf("Woke %i\n", woken);
+	} else if (strcmp(argv[3], "upfair") == 0) {
+		int woken;
+		woken = futex_up_fair(futex);
+		if (woken < 0) {
+			perror("futex_up");
+			exit(1);
+		}
+		printf("Woke %i\n", woken);
 	} else if (strcmp(argv[3], "down") == 0) {
 		if (futex_down(futex) != 0) {
 			perror("futex_down");
 			exit(1);
 		}
+	} else if (strcmp(argv[3], "down1sec") == 0) {
+		struct timespec t;
+		t.tv_sec = 1;
+		t.tv_nsec = 0;
+		if (futex_down_timeout(futex, &t) != 0) {
+			perror("futex_down_timeout");
+			exit(1);
+		}
 	} else if (strcmp(argv[3], "mem") == 0) {
 		struct futex myfutex;
 		futex_init(&myfutex);
