@@ -11,18 +11,17 @@
 /* This must include the futex call */
 #include <asm/unistd.h>
 
-/* FIXME: Standby code in case no real unistd.h */
-#ifndef __NR_futex
-#ifdef __PPC
-#define __NR_futex		208
+#ifndef PROT_SEM
 #define PROT_SEM		0x08
-#else
-#define __NR_futex		239
-#define PROT_SEM		0x08
-#endif
 #endif
 
-static _syscall2(int,futex,struct futex *,futex,int, op);
+#define __NR_sys_futex __NR_futex
+
+_syscall4(int,sys_futex,
+	  int *, futex,
+	  int, op,
+	  int, val,
+	  struct timespec *, rel);
 
 int futex_region(void *area, size_t size)
 {
@@ -37,14 +36,32 @@ int futex_region(void *area, size_t size)
 	return mprotect(area, size, PROT_READ|PROT_WRITE|PROT_SEM);
 }
 
+/* Returns -1 on fail, 0 on wakeup, 1 on pass, 2 on didn't sleep */
+int __futex_down_slow(struct futex *futx, int val, struct timespec *rel)
+{
+	if (sys_futex(&futx->count, FUTEX_WAIT, val, rel) == 0) {
+		/* <= in case someone else decremented it */
+		if (futx->count <= FUTEX_PASSED) {
+			futx->count = -1;
+			return 1;
+		}
+		return 0;
+	}
+	/* EWOULDBLOCK just means value changed before we slept: loop */
+	if (errno == EWOULDBLOCK)
+		return 2;
+	return -1;
+}
+
+int __futex_up_slow(struct futex *futx)
+{
+	futx->count = 1;
+	__futex_commit();
+	return sys_futex(&futx->count, FUTEX_WAKE, 1, NULL);
+}
+
 void futex_init(struct futex *futx)
 {
 	futx->count = 1;
 	__futex_commit();
 }
-
-/* Down and up operations in kernel */
-int sys_futex(struct futex *futx, int op)
-{
-	return futex(futx, op);
-}
